@@ -290,7 +290,7 @@
   (lambda (lst)
 	(mapcar 
 	 (lambda (str)
-	   (prime-string-split str "="))
+	   (string-split str "="))
 	 lst)))
 
 (define prime-commit-candidate
@@ -323,8 +323,8 @@
   (lambda (cands-string)
 	(mapcar
 	 (lambda (str-line)
-	   (prime-string-split str-line "\t"))
-	 (cdr (delq "" (prime-string-split cands-string "\n"))))))
+	   (string-split str-line "\t"))
+	 (cdr (delq "" (string-split cands-string "\n"))))))
 
 (define prime-begin-conversion
   (lambda (sc id)
