@@ -1,5 +1,7 @@
 ;; SKK is a Japanese input method
 ;;
+;; EUC-JP
+;;
 ;; SKKϤϲξ֤ǹ
 ;; Following is list of SKK input state
 ;;  ľ direct
@@ -25,8 +27,6 @@
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
 (define skk-use-recursive-learning? #f)
-; 47 = '/'
-(define skk-latin-conv-key 47)
 ;; key defs
 (define skk-latin-key
   (lambda (key key-state)
@@ -49,10 +49,10 @@
 (define skk-commit-key
   (lambda (key key-state)
     (generic-commit-key key key-state)))
-(define skk-next-candidate
+(define skk-next-candidate-key
   (lambda (key key-state)
     (generic-next-candidate-key key key-state)))
-(define skk-prev-candidate
+(define skk-prev-candidate-key
   (lambda (key key-state)
     (or
      (generic-prev-candidate-key key key-state)
@@ -68,6 +68,15 @@
 (define skk-backspace-key
   (lambda (key key-state)
     (generic-backspace-key key key-state)))
+; 47 = '/'
+(define skk-latin-conv-key
+  (lambda (key key-state)
+    (= key 47)))
+; Q
+(define skk-kanji-mode-key
+  (lambda (key key-state)
+    (= key 81)))
+
 
 ;; access
 (define skk-context-latin-conv
@@ -290,11 +299,25 @@
 ;; commit string
 (define skk-commit
   (lambda (sc id str)
-    (if (skk-context-parent-context sc)
-	(skk-editor-commit
-	 (skk-context-editor (skk-context-parent-context sc))
-	 str)
-	(im-commit id str))))
+    (let ((psc (skk-context-parent-context sc)))
+      (if psc
+	  (skk-editor-commit
+	   (skk-context-editor psc) str)
+	  (im-commit id str)))))
+
+(define skk-append-string
+  (lambda (sc str)
+    (and
+     str
+     (if (not (string? (car str)))
+	 (begin
+	   (skk-append-string sc (car str))
+	   (skk-append-string sc (cdr str))
+	   )
+	 #t)
+     (skk-context-set-head!
+      sc
+      (cons str (skk-context-head sc))))))
 
 (define skk-begin-conversion
   (lambda (sc id)
@@ -303,8 +326,7 @@
       (if (= (skk-context-state sc) 'skk-state-kanji)
  	  (let ((tmp (rk-push-key-last! (skk-context-rk-context sc))))
  	    (if tmp
- 		(skk-context-set-head! sc
- 				       (cons tmp (skk-context-head sc))))))
+ 		(skk-append-string sc tmp))))
       ;;
       (set! res
 	    (skk-lib-get-entry
@@ -415,6 +437,20 @@
 	   (skk-update-prop-label sc id)
 	   #f)
 	 #t)
+     (if (skk-latin-conv-key key key-state)
+	 (begin
+	   (skk-context-set-state! sc 'skk-state-kanji)
+	   (skk-update-mode id sc)
+	   (skk-context-set-latin-conv! sc #t)
+	   #f)
+	 #t)
+     (if (skk-kanji-mode-key key key-state)
+	 (begin
+	   (skk-context-set-state! sc 'skk-state-kanji)
+	   (skk-update-mode id sc)
+	   (skk-context-set-latin-conv! sc #f)
+	   #f)
+	 #t)
      (if (skk-kana-toggle key key-state)
 	 (begin 
 	   (skk-context-kana-toggle sc)
@@ -433,6 +469,7 @@
 	   (res))
       (set! res nil)
       (and
+       ;; at first, no preedit mode
        (if (string-equal? (rk-pending rkc) "")
 	   (skk-proc-state-direct-no-preedit key key-state id sc rkc)
 	   #t)
@@ -455,15 +492,6 @@
 	     (skk-commit-raw sc id key key-state)
 	     #f)
 	   #t)
-       (if (and
-	    (string-equal? (rk-pending rkc) "")
-	    (= key skk-latin-conv-key))
-	   (begin
-	     (skk-context-set-state! sc 'skk-state-kanji)
-	     (skk-update-mode id sc)
-	     (skk-context-set-latin-conv! sc #t)
-	     #f)
-	   #t)
        (if (and
 	    (shift-key-mask key-state)
 	    (alphabet-char? key))
@@ -561,10 +589,17 @@
 					  (charcode->string key))
 	     (let ((tmp (rk-push-key-last! (skk-context-rk-context sc))))
 	       (if tmp
-		   (skk-context-set-head! sc
-					  (cons tmp (skk-context-head sc)))))
+		   (skk-append-string sc tmp)))
 	     #t)
 	   #t)
+       (if (skk-context-latin-conv sc)
+	   (begin
+	     (if (alphabet-char? key)
+		 (let* ((s (charcode->string (to-lower-char key)))
+			(p (cons s s)))
+		   (skk-append-string sc p)))
+	     #f)
+	   #t)
        (begin
 	 (set! key (to-lower-char key))  
 	 (set! stat (skk-context-state sc))
@@ -656,10 +691,10 @@
 	  (id (context-id c))
 	  (res ()))
       (and
-       (if (skk-next-candidate key key-state)
+       (if (skk-next-candidate-key key key-state)
 	   (skk-change-candidate-index sc id #t)
 	   #t)
-       (if (skk-prev-candidate key key-state)
+       (if (skk-prev-candidate-key key key-state)
 	   (skk-change-candidate-index sc id #f)
 	   #t)
        (if (skk-cancel-key key key-state)
