@@ -357,14 +357,10 @@ compose_line_parts(struct dic_info *di, struct skk_line *sl,
 
 }
 
-/*
- * Compose skk line
- */
 static struct skk_line *
-compose_line(struct dic_info *di, char *word, char okuri_head, char *entry)
+alloc_skk_line(char *word, char okuri_head)
 {
   struct skk_line *sl;
-
   sl = malloc(sizeof(struct skk_line));
   sl->need_save = 0;
   sl->head = strdup(word);
@@ -377,6 +373,18 @@ compose_line(struct dic_info *di, char *word, char okuri_head, char *entry)
   sl->cands[0].nr_real_cands = 0;
   sl->cands[0].is_used = 0;
   sl->cands[0].line = sl;
+  return sl;
+}
+
+/*
+ * Compose skk line
+ */
+static struct skk_line *
+compose_line(struct dic_info *di, char *word, char okuri_head, char *entry)
+{
+  struct skk_line *sl;
+
+  sl = alloc_skk_line(word, okuri_head);
 
   /* parse */
   compose_line_parts(di, sl, NULL, entry);
@@ -447,7 +455,8 @@ skk_search_line_from_cache(struct dic_info *di, char *s, char okuri_head)
 
 static struct skk_cand_array *
 find_cand_array(struct dic_info *di, char *s,
-		char okuri_head, char *okuri)
+		char okuri_head, char *okuri,
+		int create_if_not_found)
 {
   struct skk_line *sl, *sl_file;
   struct skk_cand_array *ca;
@@ -457,13 +466,17 @@ find_cand_array(struct dic_info *di, char *s,
   if (!sl) {
     sl = skk_search_line_from_file(skk_dic, s, okuri_head);
     if (!sl) {
-      return NULL;
+      if (!create_if_not_found) {
+	return NULL;
+      }
+      /**/
+      sl = alloc_skk_line(s, okuri_head);
     }
     from_file = 1;
     add_line_to_cache(di, sl);
   }
 
-  ca = find_candidate_array_from_line(sl, okuri, 0);
+  ca = find_candidate_array_from_line(sl, okuri, create_if_not_found);
 
   if (!ca->is_used) {
     /* incorporate okuri-nasi entry */
@@ -500,7 +513,7 @@ find_cand_array_lisp(LISP head_, LISP okuri_head_, LISP okuri_,
     o = os[0];
   }
 
-  ca = find_cand_array(skk_dic, hs, o, okuri);
+  ca = find_cand_array(skk_dic, hs, o, okuri, create_if_not_found);
   free(okuri);
   return ca;
 }
@@ -613,22 +626,45 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
 static void
 learn_word_to_cand_array(struct skk_cand_array *ca, char *word)
 {
+  int i, nth = -1;
+  for (i = 0; i < ca->nr_cands; i++) {
+    if (!strcmp(word, ca->cands[i])) {
+      nth = i;
+    }
+  }
+  if (nth == -1) {
+    push_back_candidate_to_array(ca, word);
+    nth = ca->nr_cands - 1;
+  }
+  reorder_candidate(ca, word);
+  ca->line->need_save = 1;
+}
+
+static char *
+sanitize_word(char *arg)
+{
+  return strdup(arg);
 }
 
 static LISP
 skk_learn_word(LISP head_, LISP okuri_head_, LISP okuri_, LISP word_)
 {
   struct skk_cand_array *ca;
-  char *word;
+  char *word, *tmp;
 
-  word = uim_get_c_string(word_);
+  tmp = uim_get_c_string(word_);
+  word = sanitize_word(tmp);
+  free(tmp);
+  if (!word) {
+    return ;
+  }
 
-  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 1);
   if (ca) {
     learn_word_to_cand_array(ca, word);
   }
   if (okuri_ != NIL) {
-    ca = find_cand_array_lisp(head_, okuri_head_, NIL, 0);
+    ca = find_cand_array_lisp(head_, okuri_head_, NIL, 1);
     if (ca) {
       learn_word_to_cand_array(ca, word);
     }
