@@ -70,6 +70,7 @@
 /*
   incoperated into libuim from gimp (Aug-02) Yusuke TABATA
   removed math functions (Oct-03) Yusuke TABATA
+  removed vms,array,thinkc functions (Oct-03) Yusuke TABATA
  */
 #include <stdio.h>
 #include <string.h>
@@ -156,17 +157,9 @@ char *user_te_readm = "";
 LISP (*user_readm) (int, struct gen_readio *) = NULL;
 LISP (*user_readt) (char *, long, int *) = NULL;
 void (*fatal_exit_hook) (void) = NULL;
-#ifdef THINK_C
-int ipoll_counter = 0;
-#endif
 
 char *stack_limit_ptr = NULL;
-long stack_size =
-#ifdef THINK_C
-10000;
-#else
-50000;
-#endif
+long stack_size = 50000;
 
 long siod_verbose_level = 4;
 
@@ -185,7 +178,7 @@ process_cla (int argc, char **argv, int warnflag)
   int k;
   char *ptr;
   static int siod_lib_set = 0;
-#if !defined(vms)
+
   if (!siod_lib_set)
     {
       if (getenv ("SIOD_LIB"))
@@ -194,7 +187,7 @@ process_cla (int argc, char **argv, int warnflag)
 	  siod_lib_set = 1;
 	}
     }
-#endif
+
   for (k = 1; k < argc; ++k)
     {
       if (strlen (argv[k]) < 2)
@@ -369,6 +362,35 @@ noprompt_puts (char *st)
 static char *repl_c_string_arg = NULL;
 static long repl_c_string_flag = 0;
 
+static int
+rfs_getc (unsigned char **p)
+{
+  int i;
+  i = **p;
+  if (!i)
+    return (EOF);
+  *p = *p + 1;
+  return (i);
+}
+
+static void
+rfs_ungetc (unsigned char c, unsigned char **p)
+{
+  *p = *p - 1;
+}
+
+static LISP
+read_from_string (LISP x)
+{
+  char *p;
+  struct gen_readio s;
+  p = get_c_string (x);
+  s.getc_fcn = (int (*)(void *)) rfs_getc;
+  s.ungetc_fcn = (void (*)(int, void *)) rfs_ungetc;
+  s.cb_argument = (char *) &p;
+  return (readtl (&s));
+}
+
 static LISP
 repl_c_string_read (void)
 {
@@ -437,44 +459,6 @@ myruntime (void)
 #endif
 }
 
-#if defined(__osf__)
-#include <sys/timers.h>
-#ifndef TIMEOFDAY
-#define TIMEOFDAY 1
-#endif
-double
-myrealtime (void)
-{
-  struct timespec x;
-  if (!getclock (TIMEOFDAY, &x))
-    return (x.tv_sec + (((double) x.tv_nsec) * 1.0e-9));
-  else
-    return (0.0);
-}
-#endif
-
-#if defined(VMS)
-#include <ssdef.h>
-#include <starlet.h>
-
-double
-myrealtime (void)
-{
-  unsigned long x[2];
-  static double c = 0.0;
-  if (sys$gettim (&x) == SS$_NORMAL)
-    {
-      if (c == 0.0)
-	c = pow ((double) 2, (double) 31) * 100.0e-9;
-      return (x[0] * 100.0e-9 + x[1] * c);
-    }
-  else
-    return (0.0);
-}
-
-#endif
-
-#if !defined(__osf__) & !defined(VMS)
 double
 myrealtime (void)
 {
@@ -482,7 +466,6 @@ myrealtime (void)
   time (&x);
   return ((double) x);
 }
-#endif
 
 void
 set_repl_hooks (void (*puts_f) (char *),
@@ -754,12 +737,8 @@ get_c_string_dim (LISP x, long *len)
       *len = strlen (PNAME (x));
       return (PNAME (x));
     case tc_string:
-    case tc_byte_array:
       *len = x->storage_as.string.dim;
       return (x->storage_as.string.data);
-    case tc_long_array:
-      *len = x->storage_as.long_array.dim * sizeof (long);
-      return ((char *) x->storage_as.long_array.data);
     default:
       my_err ("not a symbol or string", x);
       return (NULL);
@@ -1696,10 +1675,6 @@ gc_mark_and_sweep (void)
   mark_protected_registers ();
   mark_locations ((LISP *) stack_start_ptr,
 		  (LISP *) & stack_end);
-#ifdef THINK_C
-  mark_locations ((LISP *) ((char *) stack_start_ptr + 2),
-		  (LISP *) ((char *) &stack_end + 2));
-#endif
   gc_sweep ();
   gc_ms_stats_end ();
 }
@@ -2801,11 +2776,6 @@ f_getc (FILE * f)
   iflag = no_interrupt (1);
   dflag = interrupt_differed;
   c = getc (f);
-#ifdef VMS
-  if ((dflag == 0) & interrupt_differed & (f == stdin))
-    while ((c != 0) & (c != EOF))
-      c = getc (f);
-#endif
   no_interrupt (iflag);
   return (c);
 }
@@ -2876,6 +2846,27 @@ set_read_hooks (char *all_set, char *end_set,
   user_readt = fcn2;
 }
 
+static LISP
+lreadsharp (struct gen_readio * f)
+{
+  LISP obj, l, result;
+  long j, n;
+  int c;
+  c = GETC_FCN (f);
+  switch (c)
+    {
+    case '.':
+      obj = lreadr (f);
+      return (leval (obj, NIL));
+    case 'f':
+      return (NIL);
+    case 't':
+      return (flocons (1));
+    default:
+      return (my_err ("readsharp syntax not handled", NIL));
+    }
+}
+
 LISP
 lreadr (struct gen_readio *f)
 {
@@ -3227,7 +3218,7 @@ vload (char *fname, long cflag, long rflag)
   return (result);
 }
 
-LISP
+static LISP
 load (LISP fname, LISP cflag, LISP rflag)
 {
   long len;
@@ -3289,7 +3280,7 @@ save_forms (LISP fname, LISP forms, LISP how)
   return (sym_t);
 }
 
-LISP
+static LISP
 quit (void)
 {
   return (my_err (NULL, NIL));
@@ -3591,7 +3582,7 @@ lllast_c_errmsg (void)
   return (llast_c_errmsg (-1));
 }
 
-LISP
+static LISP
 help (void)
 {
   fprintf (siod_output, "HELP for SIOD, Version %s\n", siod_version ());
@@ -3747,6 +3738,7 @@ init_subrs_1 (void)
   init_subr_0 ("help", help);
   init_subr_1 ("feature?", featurep);
   init_subr_1 ("provide", provide);
+  init_subr_1 ("read-from-string", read_from_string);
   init_slib_version ();
 }
 
