@@ -32,6 +32,10 @@ static struct {
   {UKey_F11, "F11"},
   {UKey_F12, "F12"},
   {UKey_Multi_key, "Multi_key"},
+  {UKey_Shift_key, "Shift_key"},
+  {UKey_Alt_key, "Alt_key"},
+  {UKey_Control_key, "Control_key"},
+  {UKey_Meta_key, "Meta_key"},
   /*  {UKey_Other, "other"},*/
   {0,0}
 };
