@@ -6,6 +6,8 @@
 extern "C" {
 #endif
 
+#include <stdio.h>
+
   typedef struct uim_context_ *uim_context;
 
   /* This will be filled later, so ABI is unstable */
@@ -143,6 +145,11 @@ int uim_init();
   int   uim_get_candidate_index(uim_context uc);
   void  uim_set_candidate_index(uim_context uc, int index);
 
+  /* Utility functions */
+  int
+  uim_ipc_open_command(int old_pid, FILE **read_handler, FILE **write_handler, char *command);
+
+
 #ifdef __cplusplus
 }
 #endif
