@@ -285,6 +285,9 @@ void InputContext::pushback_preedit_string(int attr, char *str)
     if (attr & UPeAttr_Cursor) {
 	m_pe->caret_pos = m_pe->get_char_count();
     }
+    if (!strlen(str)) {
+	return;
+    }
     m_pe->new_segment(p);
     uString js;
     str_to_ustring(&js, str);
